@@ -28,7 +28,7 @@
 #ifndef __GLEWLWYD_H_
 #define __GLEWLWYD_H_
 
-#define _GLEWLWYD_VERSION_ "2.7.1"
+#define _GLEWLWYD_VERSION_ "2.7.5"
 
 #include <jansson.h>
 
@@ -106,52 +106,58 @@
 #define GLEWLWYD_MODULE_ACTION_START 1
 
 // Environment variables names
-#define GLEWLWYD_ENV_PORT                        "GLWD_PORT"
-#define GLEWLWYD_ENV_MAX_POST_SIZE               "GLWD_MAX_POST_SIZE"
-#define GLEWLWYD_ENV_BIND_ADDRESS                "GLWD_BIND_ADDRESS"
-#define GLEWLWYD_ENV_API_PREFIX                  "GLWD_API_PREFIX"
-#define GLEWLWYD_ENV_EXTERNAL_URL                "GLWD_EXTERNAL_URL"
-#define GLEWLWYD_ENV_LOGIN_URL                   "GLWD_LOGIN_URL"
-#define GLEWLWYD_ENV_PROFILE_DELETE              "GLWD_PROFILE_DELETE"
-#define GLEWLWYD_ENV_STATIC_FILES_PATH           "GLWD_STATIC_FILES_PATH"
-#define GLEWLWYD_ENV_STATIC_FILES_MIME_TYPES     "GLWD_STATIC_FILES_MIME_TYPES"
-#define GLEWLWYD_ENV_ALLOW_ORIGIN                "GLWD_ALLOW_ORIGIN"
-#define GLEWLWYD_ENV_ALLOW_METHODS               "GLWD_ALLOW_METHODS"
-#define GLEWLWYD_ENV_ALLOW_HEADERS               "GLWD_ALLOW_HEADERS"
-#define GLEWLWYD_ENV_EXPOSE_HEADERS              "GLWD_EXPOSE_HEADERS"
-#define GLEWLWYD_ENV_LOG_MODE                    "GLWD_LOG_MODE"
-#define GLEWLWYD_ENV_LOG_LEVEL                   "GLWD_LOG_LEVEL"
-#define GLEWLWYD_ENV_LOG_FILE                    "GLWD_LOG_FILE"
-#define GLEWLWYD_ENV_COOKIE_DOMAIN               "GLWD_COOKIE_DOMAIN"
-#define GLEWLWYD_ENV_COOKIE_SECURE               "GLWD_COOKIE_SECURE"
-#define GLEWLWYD_ENV_COOKIE_SAME_SITE            "GLWD_COOKIE_SAME_SITE"
-#define GLEWLWYD_ENV_ADD_X_FRAME_DENY            "GLWD_ADD_X_FRAME_DENY"
-#define GLEWLWYD_ENV_SESSION_EXPIRATION          "GLWD_SESSION_EXPIRATION"
-#define GLEWLWYD_ENV_SESSION_KEY                 "GLWD_SESSION_KEY"
-#define GLEWLWYD_ENV_ADMIN_SCOPE                 "GLWD_ADMIN_SCOPE"
-#define GLEWLWYD_ENV_PROFILE_SCOPE               "GLWD_PROFILE_SCOPE"
-#define GLEWLWYD_ENV_USER_MODULE_PATH            "GLWD_USER_MODULE_PATH"
-#define GLEWLWYD_ENV_USER_MIDDLEWARE_MODULE_PATH "GLWD_USER_MIDDLEWARE_MODULE_PATH"
-#define GLEWLWYD_ENV_CLIENT_MODULE_PATH          "GLWD_CLIENT_MODULE_PATH"
-#define GLEWLWYD_ENV_AUTH_SCHEME_MODULE_PATH     "GLWD_AUTH_SCHEME_MODULE_PATH"
-#define GLEWLWYD_ENV_PLUGIN_MODULE_PATH          "GLWD_PLUGIN_MODULE_PATH"
-#define GLEWLWYD_ENV_USE_SECURE_CONNECTION       "GLWD_USE_SECURE_CONNECTION"
-#define GLEWLWYD_ENV_SECURE_CONNECTION_KEY_FILE  "GLWD_SECURE_CONNECTION_KEY_FILE"
-#define GLEWLWYD_ENV_SECURE_CONNECTION_PEM_FILE  "GLWD_SECURE_CONNECTION_PEM_FILE"
-#define GLEWLWYD_ENV_SECURE_CONNECTION_CA_FILE   "GLWD_SECURE_CONNECTION_CA_FILE"
-#define GLEWLWYD_ENV_HASH_ALGORITHM              "GLWD_HASH_ALGORITHM"
-#define GLEWLWYD_ENV_DATABASE_TYPE               "GLWD_DATABASE_TYPE"
-#define GLEWLWYD_ENV_DATABASE_MARIADB_HOST       "GLWD_DATABASE_MARIADB_HOST"
-#define GLEWLWYD_ENV_DATABASE_MARIADB_USER       "GLWD_DATABASE_MARIADB_USER"
-#define GLEWLWYD_ENV_DATABASE_MARIADB_PASSWORD   "GLWD_DATABASE_MARIADB_PASSWORD"
-#define GLEWLWYD_ENV_DATABASE_MARIADB_DBNAME     "GLWD_DATABASE_MARIADB_DBNAME"
-#define GLEWLWYD_ENV_DATABASE_MARIADB_PORT       "GLWD_DATABASE_MARIADB_PORT"
-#define GLEWLWYD_ENV_DATABASE_SQLITE3_PATH       "GLWD_DATABASE_SQLITE3_PATH"
-#define GLEWLWYD_ENV_DATABASE_POSTGRE_CONNINFO   "GLWD_DATABASE_POSTGRE_CONNINFO"
-#define GLEWLWYD_ENV_METRICS                     "GLWD_METRICS"
-#define GLEWLWYD_ENV_METRICS_PORT                "GLWD_METRICS_PORT"
-#define GLEWLWYD_ENV_METRICS_ADMIN               "GLWD_METRICS_ADMIN"
-#define GLEWLWYD_ENV_METRICS_BIND_ADDRESS        "GLWD_METRICS_BIND_ADDRESS"
+#define GLEWLWYD_ENV_PORT                         "GLWD_PORT"
+#define GLEWLWYD_ENV_MAX_POST_SIZE                "GLWD_MAX_POST_SIZE"
+#define GLEWLWYD_ENV_BIND_ADDRESS                 "GLWD_BIND_ADDRESS"
+#define GLEWLWYD_ENV_API_PREFIX                   "GLWD_API_PREFIX"
+#define GLEWLWYD_ENV_EXTERNAL_URL                 "GLWD_EXTERNAL_URL"
+#define GLEWLWYD_ENV_LOGIN_URL                    "GLWD_LOGIN_URL"
+#define GLEWLWYD_ENV_PROFILE_DELETE               "GLWD_PROFILE_DELETE"
+#define GLEWLWYD_ENV_STATIC_FILES_PATH            "GLWD_STATIC_FILES_PATH"
+#define GLEWLWYD_ENV_STATIC_FILES_MIME_TYPES      "GLWD_STATIC_FILES_MIME_TYPES"
+#define GLEWLWYD_ENV_ALLOW_ORIGIN                 "GLWD_ALLOW_ORIGIN"
+#define GLEWLWYD_ENV_ALLOW_METHODS                "GLWD_ALLOW_METHODS"
+#define GLEWLWYD_ENV_ALLOW_HEADERS                "GLWD_ALLOW_HEADERS"
+#define GLEWLWYD_ENV_EXPOSE_HEADERS               "GLWD_EXPOSE_HEADERS"
+#define GLEWLWYD_ENV_LOG_MODE                     "GLWD_LOG_MODE"
+#define GLEWLWYD_ENV_LOG_LEVEL                    "GLWD_LOG_LEVEL"
+#define GLEWLWYD_ENV_LOG_FILE                     "GLWD_LOG_FILE"
+#define GLEWLWYD_ENV_COOKIE_DOMAIN                "GLWD_COOKIE_DOMAIN"
+#define GLEWLWYD_ENV_COOKIE_SECURE                "GLWD_COOKIE_SECURE"
+#define GLEWLWYD_ENV_COOKIE_SAME_SITE             "GLWD_COOKIE_SAME_SITE"
+#define GLEWLWYD_ENV_ADD_X_FRAME_DENY             "GLWD_ADD_X_FRAME_DENY"
+#define GLEWLWYD_ENV_SESSION_EXPIRATION           "GLWD_SESSION_EXPIRATION"
+#define GLEWLWYD_ENV_SESSION_KEY                  "GLWD_SESSION_KEY"
+#define GLEWLWYD_ENV_ADMIN_SCOPE                  "GLWD_ADMIN_SCOPE"
+#define GLEWLWYD_ENV_PROFILE_SCOPE                "GLWD_PROFILE_SCOPE"
+#define GLEWLWYD_ENV_USER_MODULE_PATH             "GLWD_USER_MODULE_PATH"
+#define GLEWLWYD_ENV_USER_MIDDLEWARE_MODULE_PATH  "GLWD_USER_MIDDLEWARE_MODULE_PATH"
+#define GLEWLWYD_ENV_CLIENT_MODULE_PATH           "GLWD_CLIENT_MODULE_PATH"
+#define GLEWLWYD_ENV_AUTH_SCHEME_MODULE_PATH      "GLWD_AUTH_SCHEME_MODULE_PATH"
+#define GLEWLWYD_ENV_PLUGIN_MODULE_PATH           "GLWD_PLUGIN_MODULE_PATH"
+#define GLEWLWYD_ENV_USE_SECURE_CONNECTION        "GLWD_USE_SECURE_CONNECTION"
+#define GLEWLWYD_ENV_SECURE_CONNECTION_KEY_FILE   "GLWD_SECURE_CONNECTION_KEY_FILE"
+#define GLEWLWYD_ENV_SECURE_CONNECTION_PEM_FILE   "GLWD_SECURE_CONNECTION_PEM_FILE"
+#define GLEWLWYD_ENV_SECURE_CONNECTION_CA_FILE    "GLWD_SECURE_CONNECTION_CA_FILE"
+#define GLEWLWYD_ENV_HASH_ALGORITHM               "GLWD_HASH_ALGORITHM"
+#define GLEWLWYD_ENV_DATABASE_TYPE                "GLWD_DATABASE_TYPE"
+#define GLEWLWYD_ENV_DATABASE_MARIADB_HOST        "GLWD_DATABASE_MARIADB_HOST"
+#define GLEWLWYD_ENV_DATABASE_MARIADB_USER        "GLWD_DATABASE_MARIADB_USER"
+#define GLEWLWYD_ENV_DATABASE_MARIADB_PASSWORD    "GLWD_DATABASE_MARIADB_PASSWORD"
+#define GLEWLWYD_ENV_DATABASE_MARIADB_DBNAME      "GLWD_DATABASE_MARIADB_DBNAME"
+#define GLEWLWYD_ENV_DATABASE_MARIADB_PORT        "GLWD_DATABASE_MARIADB_PORT"
+#define GLEWLWYD_ENV_DATABASE_SQLITE3_PATH        "GLWD_DATABASE_SQLITE3_PATH"
+#define GLEWLWYD_ENV_DATABASE_POSTGRE_CONNINFO    "GLWD_DATABASE_POSTGRE_CONNINFO"
+#define GLEWLWYD_ENV_METRICS                      "GLWD_METRICS"
+#define GLEWLWYD_ENV_METRICS_PORT                 "GLWD_METRICS_PORT"
+#define GLEWLWYD_ENV_METRICS_ADMIN                "GLWD_METRICS_ADMIN"
+#define GLEWLWYD_ENV_METRICS_BIND_ADDRESS         "GLWD_METRICS_BIND_ADDRESS"
+#define GLEWLWYD_ENV_RESPONSE_ALLOWED_COMPRESSION "GLWD_RESPONSE_ALLOWED_COMPRESSION"
+#define GLEWLWYD_ENV_ADMIN_SESSION_AUTH           "GLWD_ADMIN_SESSION_AUTH"
+#define GLEWLWYD_ENV_PROFILE_SESSION_AUTH         "GLWD_PROFILE_SESSION_AUTH"
+#define GLEWLWYD_ENV_MULTIPLE_USER_SESSION        "GLWD_MULTIPLE_USER_SESSION"
+#define GLEWLWYD_ENV_LOGIN_API_ENABLED            "GLWD_LOGIN_API_ENABLED"
+#define GLEWLWYD_ENV_PLUGIN_API_RUN_ENABLED       "GLWD_PLUGIN_API_RUN_ENABLED"
 
 struct send_mail_content_struct {
   char                   * host;
@@ -200,6 +206,7 @@ char * get_ip_data(struct config_elements * config, const char * ip_address);
 const char * get_template_property(json_t * j_params, const char * template_property, const char * user_lang, const char * property_field);
 char * complete_template(const char * template, ...);
 void * thread_send_mail(void * args);
+int is_plugin_api_run_enabled (struct config_elements * config, const char * name);
 
 // Modules generic functions
 int module_parameters_check(const char * module_parameters);
@@ -220,6 +227,7 @@ json_t * get_session_for_username(struct config_elements * config, const char *
 json_t * get_current_user_for_session(struct config_elements * config, const char * session_uid);
 json_t * get_users_for_session(struct config_elements * config, const char * session_uid);
 int user_session_delete(struct config_elements * config, const char * session_uid, const char * username);
+char * get_valid_session_id(struct config_elements * config, const struct _u_request * request, const char * username);
 char * get_session_id(struct config_elements * config, const struct _u_request * request);
 char * generate_session_id();
 json_t * get_user_session_list(struct config_elements * config, const char * username, const char * pattern, size_t offset, size_t limit, const char * sort);
@@ -506,5 +514,6 @@ int callback_metrics (const struct _u_request * request, struct _u_response * re
 
 int callback_default (const struct _u_request * request, struct _u_response * response, void * user_data);
 int callback_404_if_necessary (const struct _u_request * request, struct _u_response * response, void * user_data);
+int callback_403_whatever_the_means (const struct _u_request * request, struct _u_response * response, void * user_data);
 
 #endif
