@@ -85,37 +85,45 @@ json_t * glewlwyd_callback_check_session_valid(struct config_plugin * config, co
   json_t * j_user, * j_return, * j_scope_allowed;
   char * session_uid = NULL;
   
-  if (config != NULL && request != NULL) {
-    session_uid = get_session_id(config->glewlwyd_config, request);
-    j_user = get_current_user_for_session(config->glewlwyd_config, session_uid);
-    // Check if session is valid
-    if (check_result_value(j_user, G_OK)) {
-      if (!o_strnullempty(scope_list)) {
-        // For all allowed scope, check that the current session has a valid session
-        j_scope_allowed = get_validated_auth_scheme_list_from_scope_list(config->glewlwyd_config, scope_list, session_uid);
-        if (check_result_value(j_scope_allowed, G_OK)) {
-          j_return = json_pack("{sis{sOsO}}", "result", G_OK, "session", "scope", json_object_get(j_scope_allowed, "scheme"), "user", json_object_get(j_user, "user"));
-        } else if (check_result_value(j_scope_allowed, G_ERROR_UNAUTHORIZED) || check_result_value(j_scope_allowed, G_ERROR_NOT_FOUND)) {
-          j_return = json_pack("{sis{sO}}", "result", G_ERROR_UNAUTHORIZED, "session", "user", json_object_get(j_user, "user"));
+  if (config->glewlwyd_config->login_api_enabled) {
+    if (config != NULL && request != NULL) {
+      session_uid = get_session_id(config->glewlwyd_config, request);
+      if (!o_strnullempty(session_uid)) {
+        j_user = get_current_user_for_session(config->glewlwyd_config, session_uid);
+        // Check if session is valid
+        if (check_result_value(j_user, G_OK)) {
+          if (!o_strnullempty(scope_list)) {
+            // For all allowed scope, check that the current session has a valid session
+            j_scope_allowed = get_validated_auth_scheme_list_from_scope_list(config->glewlwyd_config, scope_list, session_uid);
+            if (check_result_value(j_scope_allowed, G_OK)) {
+              j_return = json_pack("{sis{sOsO}}", "result", G_OK, "session", "scope", json_object_get(j_scope_allowed, "scheme"), "user", json_object_get(j_user, "user"));
+            } else if (check_result_value(j_scope_allowed, G_ERROR_UNAUTHORIZED) || check_result_value(j_scope_allowed, G_ERROR_NOT_FOUND)) {
+              j_return = json_pack("{sis{sO}}", "result", G_ERROR_UNAUTHORIZED, "session", "user", json_object_get(j_user, "user"));
+            } else {
+              y_log_message(Y_LOG_LEVEL_ERROR, "glewlwyd_callback_check_session_valid - Error get_validated_auth_scheme_list_from_scope_list");
+              j_return = json_pack("{si}", "result", G_ERROR);
+            }
+            json_decref(j_scope_allowed);
+          } else {
+            j_return = json_pack("{sis{sO}}", "result", G_OK, "session", "user", json_object_get(j_user, "user"));
+          }
+        } else if (check_result_value(j_user, G_ERROR_NOT_FOUND)) {
+          j_return = json_pack("{si}", "result", G_ERROR_NOT_FOUND);
         } else {
-          y_log_message(Y_LOG_LEVEL_ERROR, "glewlwyd_callback_check_session_valid - Error get_validated_auth_scheme_list_from_scope_list");
+          y_log_message(Y_LOG_LEVEL_ERROR, "glewlwyd_callback_check_session_valid - Error get_current_user_for_session");
           j_return = json_pack("{si}", "result", G_ERROR);
         }
-        json_decref(j_scope_allowed);
+        json_decref(j_user);
       } else {
-        j_return = json_pack("{sis{sO}}", "result", G_OK, "session", "user", json_object_get(j_user, "user"));
+        j_return = json_pack("{si}", "result", G_ERROR_UNAUTHORIZED);
       }
-    } else if (check_result_value(j_user, G_ERROR_NOT_FOUND)) {
-      j_return = json_pack("{si}", "result", G_ERROR_NOT_FOUND);
+      o_free(session_uid);
     } else {
-      y_log_message(Y_LOG_LEVEL_ERROR, "glewlwyd_callback_check_session_valid - Error get_current_user_for_session");
-      j_return = json_pack("{si}", "result", G_ERROR);
+      j_return = json_pack("{si}", "result", G_ERROR_PARAM);
     }
-    json_decref(j_user);
   } else {
-    j_return = json_pack("{si}", "result", G_ERROR_PARAM);
+    j_return = json_pack("{si}", "result", G_ERROR_UNAUTHORIZED);
   }
-  o_free(session_uid);
   return j_return;
 }
 
@@ -125,7 +133,7 @@ json_t * glewlwyd_callback_check_user_valid(struct config_plugin * config, const
   char ** scope_array = NULL, * scope_list = NULL, * tmp;
   size_t index;
 
-  if (config != NULL && username != NULL) {
+  if (config != NULL && username != NULL && config->glewlwyd_config->login_api_enabled) {
     j_user = get_user(config->glewlwyd_config, username, NULL);
     if (check_result_value(j_user, G_OK)) {
       check_password = 1;
@@ -364,8 +372,7 @@ time_t glewlwyd_callback_get_session_age(struct config_plugin * config, const st
   json_t * j_result;
   
   if (session_uid != NULL) {
-    session_uid_hash = generate_hash(config->glewlwyd_config->hash_algorithm, session_uid);
-    if (session_uid_hash != NULL) {
+    if ((session_uid_hash = generate_hash(config->glewlwyd_config->hash_algorithm, session_uid)) != NULL) {
       if (split_string(scope_list, " ", &scope_array)) {
         for (int i=0; scope_array[i] != NULL; i++) {
           scope_escaped = h_escape_string_with_quotes(config->glewlwyd_config->conn, scope_array[i]);
@@ -504,7 +511,7 @@ int glewlwyd_plugin_callback_set_user(struct config_plugin * config, const char
   if (check_result_value(j_cur_user, G_OK)) {
     ret = set_user(config->glewlwyd_config, username, j_user, json_string_value(json_object_get(json_object_get(j_cur_user, "user"), "source")));
   } else if (check_result_value(j_cur_user, G_ERROR_NOT_FOUND)) {
-    ret = U_ERROR_NOT_FOUND;
+    ret = G_ERROR_NOT_FOUND;
   } else {
     y_log_message(Y_LOG_LEVEL_DEBUG, "glewlwyd_plugin_callback_set_user - Error get_user");
     ret = G_ERROR;
@@ -514,20 +521,25 @@ int glewlwyd_plugin_callback_set_user(struct config_plugin * config, const char
 }
 
 int glewlwyd_plugin_callback_user_update_password(struct config_plugin * config, const char * username, const char * password) {
-  json_t * j_user = get_user(config->glewlwyd_config, username, NULL);
+  json_t * j_user;
   int ret;
   const char *passwords[1];
   
-  passwords[0] = password;
-  if (check_result_value(j_user, G_OK)) {
-    ret = user_set_password(config->glewlwyd_config, username, passwords, 1);
-  } else if (check_result_value(j_user, G_ERROR_NOT_FOUND)) {
-    ret = U_ERROR_NOT_FOUND;
+  if (config->glewlwyd_config->login_api_enabled) {
+    j_user = get_user(config->glewlwyd_config, username, NULL);
+    passwords[0] = password;
+    if (check_result_value(j_user, G_OK)) {
+      ret = user_set_password(config->glewlwyd_config, username, passwords, 1);
+    } else if (check_result_value(j_user, G_ERROR_NOT_FOUND)) {
+      ret = G_ERROR_NOT_FOUND;
+    } else {
+      y_log_message(Y_LOG_LEVEL_DEBUG, "glewlwyd_plugin_callback_user_update_password - Error get_user");
+      ret = G_ERROR;
+    }
+    json_decref(j_user);
   } else {
-    y_log_message(Y_LOG_LEVEL_DEBUG, "glewlwyd_plugin_callback_user_update_password - Error get_user");
-    ret = G_ERROR;
+    ret = G_ERROR_PARAM;
   }
-  json_decref(j_user);
   return ret;
 }
 
@@ -538,7 +550,7 @@ int glewlwyd_plugin_callback_delete_user(struct config_plugin * config, const ch
   if (check_result_value(j_user, G_OK)) {
     ret = delete_user(config->glewlwyd_config, username, json_string_value(json_object_get(json_object_get(j_user, "user"), "source")));
   } else if (check_result_value(j_user, G_ERROR_NOT_FOUND)) {
-    ret = U_ERROR_NOT_FOUND;
+    ret = G_ERROR_NOT_FOUND;
   } else {
     y_log_message(Y_LOG_LEVEL_DEBUG, "glewlwyd_plugin_callback_set_user - Error get_user");
     ret = G_ERROR;
