@@ -50,7 +50,7 @@ static char * escape_ldap(const char * input) {
     to_return = strdup("");
     len = o_strlen(input);
     for (i=0; i < len && to_return != NULL; i++) {
-      unsigned char c = input[i];
+      unsigned char c = (unsigned char)input[i];
       if (c == '*') {
         // escape asterisk
         tmp = msprintf("%s\\2a", to_return);
@@ -488,6 +488,7 @@ static int set_update_password_mod(json_t * j_params, LDAP * ldap, const char *
   int ldap_result, nb_values, ret = G_OK, i;
   struct berval ** result_values = NULL;
   size_t counter;
+  char * escaped = escape_ldap(username);
 
   int  scope = LDAP_SCOPE_ONELEVEL;
   char * filter = NULL;
@@ -501,13 +502,13 @@ static int set_update_password_mod(json_t * j_params, LDAP * ldap, const char *
   }
   if (!add) {
     // Connection successful, doing ldap search
-    filter = msprintf("(&(%s)(%s=%s))", json_string_value(json_object_get(j_params, "filter")), get_read_property(j_params, "username-property"), username);
+    filter = msprintf("(&(%s)(%s=%s))", json_string_value(json_object_get(j_params, "filter")), get_read_property(j_params, "username-property"), escaped);
     if ((ldap_result = ldap_search_ext_s(ldap, json_string_value(json_object_get(j_params, "base-search")), scope, filter, attrs, attrsonly, NULL, NULL, NULL, LDAP_NO_LIMIT, &answer)) != LDAP_SUCCESS) {
       y_log_message(Y_LOG_LEVEL_ERROR, "set_update_password_mod - Error ldap search, base search: %s, filter: %s: %s", json_string_value(json_object_get(j_params, "base-search")), filter, ldap_err2string(ldap_result));
       ret = G_ERROR;
     } else {
       // Looping in results, staring at offset, until the end of the list
-      if (ldap_count_entries(ldap, answer) > 0) {
+      if (ldap_count_entries(ldap, answer) == 1) {
         entry = ldap_first_entry(ldap, answer);
         result_values = ldap_get_values_len(ldap, entry, json_string_value(json_object_get(j_params, "password-property")));
         nb_values = ldap_count_values_len(result_values);
@@ -555,6 +556,7 @@ static int set_update_password_mod(json_t * j_params, LDAP * ldap, const char *
       ret = G_ERROR_MEMORY;
     }
   }
+  o_free(escaped);
   return ret;
 }
 
@@ -1150,7 +1152,7 @@ static json_t * get_user_from_result(json_t * j_params, json_t * j_properties_us
 }
 
 static char * get_user_dn_from_username(json_t * j_params, LDAP * ldap, const char * username) {
-  char * user_dn, * filter;
+  char * user_dn, * filter, * escaped = escape_ldap(username);
   int  result;
   char * attrs[] = {NULL};
   int  attrsonly = 0;
@@ -1163,11 +1165,11 @@ static char * get_user_dn_from_username(json_t * j_params, LDAP * ldap, const ch
   } else if (0 == o_strcmp(json_string_value(json_object_get(j_params, "search-scope")), "subtree")) {
     scope = LDAP_SCOPE_CHILDREN;
   }
-  filter = msprintf("(&(%s)(%s=%s))", json_string_value(json_object_get(j_params, "filter")), get_read_property(j_params, "username-property"), username);
+  filter = msprintf("(&(%s)(%s=%s))", json_string_value(json_object_get(j_params, "filter")), get_read_property(j_params, "username-property"), escaped);
   if ((result = ldap_search_ext_s(ldap, json_string_value(json_object_get(j_params, "base-search")), scope, filter, attrs, attrsonly, NULL, NULL, NULL, LDAP_NO_LIMIT, &answer)) != LDAP_SUCCESS) {
     y_log_message(Y_LOG_LEVEL_ERROR, "get_user_dn_from_username - Error ldap search, base search: %s, filter, error message: %s: %s", json_string_value(json_object_get(j_params, "base-search")), filter, ldap_err2string(result));
   } else {
-    if (ldap_count_entries(ldap, answer) > 0) {
+    if (ldap_count_entries(ldap, answer) == 1) {
       entry = ldap_first_entry(ldap, answer);
       user_dn = ldap_get_dn(ldap, entry);
       str_result = o_strdup(user_dn);
@@ -1178,6 +1180,7 @@ static char * get_user_dn_from_username(json_t * j_params, LDAP * ldap, const ch
     ldap_msgfree(answer);
   }
   o_free(filter);
+  o_free(escaped);
   return str_result;
 }
 
@@ -1246,7 +1249,7 @@ size_t user_module_count_total(struct config_module * config, const char * patte
     if ((result = ldap_search_ext_s(ldap, json_string_value(json_object_get(j_params, "base-search")), scope, filter, attrs, attrsonly, NULL, NULL, NULL, LDAP_NO_LIMIT, &answer)) != LDAP_SUCCESS) {
       y_log_message(Y_LOG_LEVEL_ERROR, "user_module_count_total ldap - Error ldap search, base search: %s, filter: %s: %s", json_string_value(json_object_get(j_params, "base-search")), filter, ldap_err2string(result));
     } else {
-      counter = ldap_count_entries(ldap, answer);
+      counter = (size_t)ldap_count_entries(ldap, answer);
     }
     ldap_msgfree(answer);
     ldap_unbind_ext(ldap, NULL, NULL);
@@ -1288,7 +1291,7 @@ json_t * user_module_get_list(struct config_module * config, const char * patter
     attrs = get_ldap_read_attributes(j_params, 0, (j_properties_user = json_object()));
     j_user_list = json_array();
     do {
-      ldap_result = ldap_create_page_control(ldap, json_integer_value(json_object_get(j_params, "page-size")), cookie, 0, &page_control);
+      ldap_result = ldap_create_page_control(ldap, (ber_int_t)json_integer_value(json_object_get(j_params, "page-size")), cookie, 0, &page_control);
       if (ldap_result != LDAP_SUCCESS) {
         y_log_message(Y_LOG_LEVEL_ERROR, "user_module_get_list ldap - Error ldap_create_page_control, message: %s", ldap_err2string(ldap_result));
         break;
@@ -1393,6 +1396,7 @@ json_t * user_module_get(struct config_module * config, const char * username, v
   LDAPMessage * entry, * answer;
   int ldap_result;
   struct berval ** result_values = NULL;
+  char * escaped = escape_ldap(username);
 
   int  scope = LDAP_SCOPE_ONELEVEL;
   char * filter = NULL;
@@ -1406,14 +1410,14 @@ json_t * user_module_get(struct config_module * config, const char * username, v
   }
   if (ldap != NULL) {
     // Connection successful, doing ldap search
-    filter = msprintf("(&(%s)(%s=%s))", json_string_value(json_object_get(j_params, "filter")), get_read_property(j_params, "username-property"), username);
+    filter = msprintf("(&(%s)(%s=%s))", json_string_value(json_object_get(j_params, "filter")), get_read_property(j_params, "username-property"), escaped);
     attrs = get_ldap_read_attributes(j_params, 0, (j_properties_user = json_object()));
     if ((ldap_result = ldap_search_ext_s(ldap, json_string_value(json_object_get(j_params, "base-search")), scope, filter, attrs, attrsonly, NULL, NULL, NULL, LDAP_NO_LIMIT, &answer)) != LDAP_SUCCESS) {
       y_log_message(Y_LOG_LEVEL_ERROR, "user_module_get user - Error ldap search, base search: %s, filter: %s: %s", json_string_value(json_object_get(j_params, "base-search")), filter, ldap_err2string(ldap_result));
       j_return = json_pack("{si}", "result", G_ERROR);
     } else {
       // Looping in results, staring at offset, until the end of the list
-      if (ldap_count_entries(ldap, answer) > 0) {
+      if (ldap_count_entries(ldap, answer) == 1) {
         entry = ldap_first_entry(ldap, answer);
         j_user = get_user_from_result(j_params, j_properties_user, ldap, entry);
         if (j_user != NULL) {
@@ -1442,6 +1446,7 @@ json_t * user_module_get(struct config_module * config, const char * username, v
     y_log_message(Y_LOG_LEVEL_ERROR, "user_module_get ldap user - Error connect_ldap_server");
     j_return = json_pack("{si}", "result", G_ERROR);
   }
+  o_free(escaped);
   return j_return;
 }
 
@@ -1452,6 +1457,7 @@ json_t * user_module_get_profile(struct config_module * config, const char * use
   LDAPMessage * entry, * answer;
   int ldap_result;
   struct berval ** result_values = NULL;
+  char * escaped = escape_ldap(username);
 
   int  scope = LDAP_SCOPE_ONELEVEL;
   char * filter = NULL;
@@ -1465,13 +1471,13 @@ json_t * user_module_get_profile(struct config_module * config, const char * use
   }
   if (ldap != NULL) {
     // Connection successful, doing ldap search
-    filter = msprintf("(&(%s)(%s=%s))", json_string_value(json_object_get(j_params, "filter")), get_read_property(j_params, "username-property"), username);
+    filter = msprintf("(&(%s)(%s=%s))", json_string_value(json_object_get(j_params, "filter")), get_read_property(j_params, "username-property"), escaped);
     attrs = get_ldap_read_attributes(j_params, 1, (j_properties_user = json_object()));
     if ((ldap_result = ldap_search_ext_s(ldap, json_string_value(json_object_get(j_params, "base-search")), scope, filter, attrs, attrsonly, NULL, NULL, NULL, LDAP_NO_LIMIT, &answer)) != LDAP_SUCCESS) {
       y_log_message(Y_LOG_LEVEL_ERROR, "user_module_get_profile ldap user - Error ldap search, base search: %s, filter: %s: %s", json_string_value(json_object_get(j_params, "base-search")), filter, ldap_err2string(ldap_result));
       j_return = json_pack("{si}", "result", G_ERROR);
     } else {
-      if (ldap_count_entries(ldap, answer) > 0) {
+      if (ldap_count_entries(ldap, answer) == 1) {
         entry = ldap_first_entry(ldap, answer);
         j_user = get_user_from_result(j_params, j_properties_user, ldap, entry);
         if (j_user != NULL) {
@@ -1500,6 +1506,7 @@ json_t * user_module_get_profile(struct config_module * config, const char * use
     y_log_message(Y_LOG_LEVEL_ERROR, "user_module_get_profile ldap user - Error connect_ldap_server");
     j_return = json_pack("{si}", "result", G_ERROR);
   }
+  o_free(escaped);
   return j_return;
 }
 
@@ -1790,9 +1797,9 @@ int user_module_check_password(struct config_module * config, const char * usern
   char * filter = NULL;
   char * attrs[] = {"memberOf", NULL, NULL};
   int attrsonly = 0;
-  char * ldap_mech = LDAP_SASL_SIMPLE;
+  char * ldap_mech = LDAP_SASL_SIMPLE, * escaped = escape_ldap(username);
   struct berval cred;
-  struct berval *servcred;
+  struct berval * servcred;
 
   if (0 == o_strcmp(json_string_value(json_object_get(j_params, "search-scope")), "subtree")) {
     scope = LDAP_SCOPE_SUBTREE;
@@ -1801,12 +1808,12 @@ int user_module_check_password(struct config_module * config, const char * usern
   }
   if (ldap != NULL) {
     // Connection successful, doing ldap search
-    filter = msprintf("(&(%s)(%s=%s))", json_string_value(json_object_get(j_params, "filter")), get_read_property(j_params, "username-property"), username);
+    filter = msprintf("(&(%s)(%s=%s))", json_string_value(json_object_get(j_params, "filter")), get_read_property(j_params, "username-property"), escaped);
     if ((ldap_result = ldap_search_ext_s(ldap, json_string_value(json_object_get(j_params, "base-search")), scope, filter, attrs, attrsonly, NULL, NULL, NULL, LDAP_NO_LIMIT, &answer)) != LDAP_SUCCESS) {
       y_log_message(Y_LOG_LEVEL_ERROR, "user_module_check_password ldap - Error ldap search, base search: %s, filter: %s: %s", json_string_value(json_object_get(j_params, "base-search")), filter, ldap_err2string(ldap_result));
       result = G_ERROR;
     } else {
-      if (ldap_count_entries(ldap, answer) > 0) {
+      if (ldap_count_entries(ldap, answer) == 1) {
         // Testing the first result to username with the given password
         entry = ldap_first_entry(ldap, answer);
         user_dn = ldap_get_dn(ldap, entry);
@@ -1831,6 +1838,7 @@ int user_module_check_password(struct config_module * config, const char * usern
     y_log_message(Y_LOG_LEVEL_ERROR, "user_module_check_password ldap - Error connect_ldap_server");
     result = G_ERROR;
   }
+  o_free(escaped);
   return result;
 }
 
