@@ -21,7 +21,8 @@ class SchemeOTP extends Component {
       otpUrl: false,
       qrcode: "",
       allowHotp: false,
-      allowTotp: false
+      allowTotp: false,
+      showForbidMessage: false
     };
     
     this.getRegister = this.getRegister.bind(this);
@@ -61,10 +62,16 @@ class SchemeOTP extends Component {
         } else {
           myOtp = res;
         }
-        this.setState({myOtp: myOtp, allowHotp: res["hotp-allow"], allowTotp: res["totp-allow"]});
+        this.setState({showForbidMessage: false, myOtp: myOtp, allowHotp: res["hotp-allow"], allowTotp: res["totp-allow"]});
       })
       .fail((err) => {
-        messageDispatcher.sendMessage('Notification', {type: "danger", message: i18next.t("error-api-connect")});
+        if (err.status === 401) {
+          messageDispatcher.sendMessage('App', {type: "loggedIn", loggedIn: false});
+        } else if (err.status === 403) {
+          this.setState({showForbidMessage: true});
+        } else {
+          messageDispatcher.sendMessage('Notification', {type: "danger", message: i18next.t("error-api-connect")});
+        }
       })
       .always(() => {
         this.showQRCode();
@@ -137,10 +144,15 @@ class SchemeOTP extends Component {
       myOtp.secret = res.secret;
       this.setState({myOtp: myOtp}, () => {
         this.showQRCode();
+        this.register();
       });
     })
     .fail((err) => {
-      messageDispatcher.sendMessage('Notification', {type: "danger", message: i18next.t("error-api-connect")});
+      if (err.status === 401) {
+        messageDispatcher.sendMessage('App', {type: "loggedIn", loggedIn: false});
+      } else {
+        messageDispatcher.sendMessage('Notification', {type: "danger", message: i18next.t("error-api-connect")});
+      }
     });
   }
   
@@ -184,14 +196,20 @@ class SchemeOTP extends Component {
           }
         })
         .fail((err) => {
-          messageDispatcher.sendMessage('Notification', {type: "danger", message: i18next.t("profile.scheme-otp-save-error")});
+          if (err.status === 400) {
+            messageDispatcher.sendMessage('Notification', {type: "danger", message: i18next.t("profile.scheme-otp-save-error")});
+          } else if (err.status === 401) {
+            messageDispatcher.sendMessage('App', {type: "loggedIn", loggedIn: false});
+          } else {
+            messageDispatcher.sendMessage('Notification', {type: "danger", message: i18next.t("error-api-connect")});
+          }
         });
       }
     });
   }
   
 	render() {
-    var jsxHOTP, jsxTOTP, secretJsx, jsxHotpOption, jsxTotpOption, jsxQrcode;
+    var jsxHOTP, jsxTOTP, secretJsx, jsxHotpOption, jsxTotpOption, jsxQrcode, jsxForbidMessage, bottomButtonJsx;
     secretJsx = 
       <div className="row">
         <div className="col-md-12">
@@ -199,9 +217,14 @@ class SchemeOTP extends Component {
             <div className="input-group-prepend">
               <span className="input-group-text">{i18next.t("profile.scheme-otp-secret")}</span>
             </div>
-            <input type="text" maxLength="128" className={!!this.state.errorList.secret?"form-control is-invalid":"form-control"} id="scheme-otp-secret" onChange={(e) => this.changeParam(e, "secret")} value={this.state.myOtp.secret} placeholder={i18next.t("profile.scheme-otp-secret-ph")} />
+            <input type="text"
+                   maxLength="128"
+                   className={!!this.state.errorList.secret?"form-control is-invalid":"form-control"}
+                   id="scheme-otp-secret" onChange={(e) => this.changeParam(e, "secret")}
+                   value={this.state.myOtp.secret}
+                   placeholder={i18next.t("profile.scheme-otp-secret-ph")} />
             <div className="input-group-append">
-              <button className="btn btn-outline-secondary" type="button" onClick={this.generateSecret}>{i18next.t("profile.scheme-otp-generate-secret")}</button>
+              <button type="button" className="btn btn-outline-secondary" onClick={this.register}>{i18next.t("profile.scheme-otp-save")}</button>
             </div>
             {!!this.state.errorList.secret?<span className="error-input">{this.state.errorList.secret}</span>:""}
           </div>
@@ -222,6 +245,7 @@ class SchemeOTP extends Component {
           </div>
         </div>
       </div>
+      bottomButtonJsx = <button className="btn btn-primary" type="button" onClick={this.generateSecret}>{i18next.t("profile.scheme-otp-generate-secret")}</button>
     } else if (this.state.myOtp.type === "TOTP") {
       jsxTOTP = <div>
         {secretJsx}
@@ -237,6 +261,9 @@ class SchemeOTP extends Component {
           </div>
         </div>
       </div>
+      bottomButtonJsx = <button className="btn btn-primary" type="button" onClick={this.generateSecret}>{i18next.t("profile.scheme-otp-generate-secret")}</button>
+    } else {
+      bottomButtonJsx = <button className="btn btn-primary" type="button" onClick={this.register}>{i18next.t("profile.scheme-otp-save")}</button>
     }
     if (this.state.allowHotp) {
       jsxHotpOption = <a className={"dropdown-item"+(this.state.myOtp.type==="HOTP"?" active":"")} href="#" onClick={(e) => this.changeType(e, "HOTP")}>{i18next.t("profile.scheme-otp-type-HOTP")}</a>;
@@ -259,6 +286,9 @@ class SchemeOTP extends Component {
           </div>
         </div>
     }
+    if (this.state.showForbidMessage) {
+      jsxForbidMessage = <h4 className="alert alert-danger">{i18next.t("profile.scheme-register-forbidden")}</h4>
+    }
 
     return (
       <div>
@@ -275,7 +305,7 @@ class SchemeOTP extends Component {
               </div>
               <div className="dropdown">
                 <button className="btn btn-secondary dropdown-toggle" type="button" id="scheme-otp-type" data-toggle="dropdown" aria-haspopup="true" aria-expanded="false">
-                  {i18next.t("profile.scheme-otp-type-" + this.state.myOtp.type)}
+                  {i18next.t("profile.scheme-otp-type-" + (this.state.myOtp.type||"NONE"))}
                 </button>
                 <div className="dropdown-menu" aria-labelledby="scheme-otp-type">
                   {jsxTotpOption}
@@ -289,6 +319,7 @@ class SchemeOTP extends Component {
         {jsxHOTP}
         {jsxTOTP}
         {jsxQrcode}
+        {jsxForbidMessage}
         <div className="row">
           <div className="col-md-12">
             <hr/>
@@ -297,7 +328,7 @@ class SchemeOTP extends Component {
         <div className="row">
           <div className="col-md-12">
             <div className="btn-group" role="group">
-              <button type="button" className="btn btn-primary" onClick={(e) => this.register(e)}>{i18next.t("profile.scheme-otp-save")}</button>
+              {bottomButtonJsx}
             </div>
           </div>
         </div>
